/* ************************************************************************** */
/*                                         _____                              */
/*  Level.cpp                             |  ___| _   ____   ___              */
/*  Project : C-Engine                    | |_	 | | |  __| / _ \             */
/*                                        |  _|	 | | | |=  |  __/             */
/*  By: Fife <wasabifife@gmail.com>       |_|	 |_| |_|    \___|  	          */
/*                                                                            */
/*                                                                            */
/* ************************************************************************** */

#include "..\Rework\include\Map\Level.hpp"

    Level::Level()
    {
        worldLink = NULL;
        listeObject = NULL;
        nbObject = 0;

        tileset = NULL;
        map = NULL;
        solid = NULL;

        tileHeight = 0;
        tileWidth = 0;
        tilegap = 0;

        levelHeight = 0;
        levelWidth = 0;

        xscroll = 0;
        yscroll = 0;
    }

    void Level::AffectWorld( World * v )
    {
        worldLink = v;
    }

    World * Level::GetWorld()
    {
        return worldLink;
    }

    void Level::SetMap( const unsigned char * userTileset ,  unsigned char * userMap , bool * userProperties, int userTileWidth , int userTileHeight , int userWorldWidth , int userWorldHeight)
    {

        tileset = userTileset;
        map = userMap;
        solid = userProperties;

        tileWidth = userTileWidth;
        tileHeight = userTileHeight;

        tilegap = userTileHeight / 8;
        if(userTileHeight % 8)tilegap++;

        levelWidth = userWorldWidth;
        levelHeight = userWorldHeight;

    }

    int Level::GetIdMap( int x , int y )
    {
        return *(map + levelWidth * y + x);
    }

    void Level::ReplaceIdMap( int x , int y , char v )
    {
        *(map + levelWidth * y + x) = v;
    }

    void Level::DrawMap()
    {
        int minx,maxx,miny,maxy;

        int dy = levelHeight * tileHeight - worldLink->GetEngine()->GetScreen()->max.y ;

        minx = worldLink->GetEngine()->GetScreen()->min.x / tileWidth ;
        miny =  dy / tileHeight ;

        maxx = 128 / tileWidth + minx + 2;
        maxy = 64 / tileHeight + miny + 2;

        xscroll = worldLink->GetEngine()->GetScreen()->min.x ;
        yscroll = dy % tileHeight;

        for( int i = minx; i <  maxx; i++)
        {
            for( int j = miny ; j < maxy; j++)
            {
                if (!(i < 0 || i >= levelWidth || j < 0 ||  j >= levelHeight))
                    ML_bmp_or_cl( tileset + tileHeight   * tilegap * GetIdMap(i,j), (i * tileWidth ) - xscroll , ((j - miny) * tileHeight) - yscroll, tileWidth , tileHeight);
            }
        }
    }

    Object ** Level::GetListeObject()
    {
        return listeObject;
    }

    int Level::GetNbObject()
    {
        return nbObject;
    }

    void Level::AddObject( Object * v)
    {
            Object ** Tab = listeObject; //On crai un pointeur sur la liste actuelle d'objet

            listeObject = (Object**) malloc( sizeof(Object *) * ( nbObject + 1 ) );//On alloue l'espace ncssaire pour le nouveau tableau

            for( int i = 0 ; i < nbObject  ; i++ )//On copie les pointeurs
                    listeObject[i] = Tab[i];

            listeObject[ nbObject ] = v; //Et on rajoute l'objet
            v->AffectEngine(GetWorld()->GetEngine());

            nbObject ++;

            delete Tab;
    }

    bool Level::DelObject( Object * v)
    {
        Object ** Tab = listeObject;
        int index = -1;

        if(!Tab)return false;

        for( int i = 0 ; i < nbObject  ; i++ )
            if(Tab[i] == v )
                index = i ;

        if(index == -1)return false;

        listeObject = (Object**) malloc( sizeof(Object *) * ( nbObject - 1) );

        for( int i = 0 ; i < index  ; i++ )
            listeObject[ i ] = Tab[ i ];

         for(int i = index + 1 ; i < nbObject  ; i++ )
            listeObject[ i - 1] = Tab[ i ];

        nbObject --;
        delete v;
        delete Tab;

        return true;
    }

    void Level::DelAllObject()
    {
        for(int i = 0 ; i < nbObject  ; i++)
            delete listeObject[i];

        delete listeObject;
        listeObject = NULL;
    }



